(function(w, x) {
    var client_info = (function() {
        var ua = navigator.userAgent
                .match(/QQHouse_For_(IOS|ANDROID)\/([\d\.]+)/);
        switch (ua && ua[1]) {
        case 'IOS':
            return {
                platform : 'i',
                version : ua[2]
            };
        case 'ANDROID':
            return {
                platform : 'a',
                version : ua[2]
            };
        default:
            return null;
        }
    })();
    if (!client_info)
        return;

    var versionCompare = function(a, b) {
        var i, cmp, len, re = /(\.0)+[^\.]*$/;
        a = (a + '').replace(re, '').split('.');
        b = (b + '').replace(re, '').split('.');
        len = Math.min(a.length, b.length);
        for (i = 0; i < len; i++) {
            cmp = parseInt(a[i], 10) - parseInt(b[i], 10);
            if (cmp !== 0) {
                return cmp;
            }
        }
        return a.length - b.length;
    };
    var versionGreaterThanEqual = function(a, b) {
        return versionCompare(a, b) >= 0;
    };
    var versionLessThan = function(a, b) {
        return versionCompare(a, b) < 0;
    };

    var callbackQueue = [];
    var encodeParams = function(params) {
        var m = params.shift();
        var ts = [];
        var l = params.length;
        for (var i = 0; i < l; i++) {
            var p = params[i];
            var t = typeof p;
            ts[ts.length] = p == null ? 'string' : t;
            if (t == 'function') {
                params[i] = callbackQueue.length;
                callbackQueue[params[i]] = p;
            }
        }
        return JSON.stringify({
            method : m,
            types : ts,
            args : params,
            instanceName : x
        })
    };

    var slice = Array.prototype.slice;

    var callNative = function() {
        var params = slice.apply(arguments);
        var platform = params.shift();
        var json = encodeParams(params);
        var src = "jsbridge://get_with_json_data?json="
                + encodeURIComponent(json);

        if (platform == 'i') {
            iframe = document.createElement("iframe");
            iframe.style.display = "none";
            iframe.src = src;
            var cleanFn = function(state) {
                try {
                    iframe.parentNode.removeChild(iframe);
                } catch (error) {
                }
            };
            iframe.onload = cleanFn;
            document.documentElement.appendChild(iframe);
        } else if (platform == 'a') {
            var t = new Image;
            t.onload = function() {
                t = null
            };
            t.src = src;
        }
    };

    var config = [
            // { name
            // iOS import version
            // iOS deprecated version
            // Android import version
            // Android deprecated version
            // doc
            // test code }
            {
                n : 'showAlert',
                iiv : '2.1',
                idv : '999',
                aiv : '2.1',
                adv : '999',
                doc : [ '{string} alertString 提醒信息',
                        '{function} callback 回调函数',
                        '{userinfo} userinfo 自定义信息：数字，字符串，数组，对象' ],
                test : [
                        "window.QQHouse.showAlert('alert from js');",
                        "window.QQHouse.showAlert('alert with callback and userinfo', window.QQHouse.democallback)",
                        "window.QQHouse.showAlert('alert with callback and userinfo', window.QQHouse.democallback, 123)",
                        "window.QQHouse.showAlert('alert with callback and userinfo', window.QQHouse.democallback, 'string')",
                        "window.QQHouse.showAlert('alert with callback and userinfo', window.QQHouse.democallback, [1,2,3])",
                        "window.QQHouse.showAlert('alert with callback and userinfo', window.QQHouse.democallback, {a:123})" ]
            },
            {
                n : 'setTitle',
                iiv : '2.1',
                idv : '999',
                aiv : '2.1',
                adv : '999',
                doc : [ '{string} title 要设置的title' ],
                test : [ "window.QQHouse.setTitle('title from js');" ]
            },
            {
                n : 'setGestureQuit',
                iiv : '2.1',
                idv : '999',
                aiv : '999',
                adv : '0',
                doc : [ '{boolean} true/false true禁止,false开启,默认开启' ],
                test : [ "window.QQHouse.setGestureQuit(true)",
                        "window.QQHouse.setGestureQuit(false)" ]
            },
            {
                n : 'setActionButton',
                iiv : '2.2',
                idv : '999',
                aiv : '2.2',
                adv : '999',
                doc : [ '{boolean} true/false true显示,false隐藏,默认显示' ],
                test : [ "window.QQHouse.setActionButton(true)",
                        "window.QQHouse.setActionButton(false)" ]
            },
            {
                n : 'setShareParams',
                iiv : '2.2',
                idv : '999',
                aiv : '2.2',
                adv : '999',
                doc : [ '{string} title 分享标题', '{string} desc 分享内容',
                        '{string} iconUrl 图标地址，默认为应为看房icon',
                        '{string} pageUrl 分享页面地址' ],
                test : [ "window.QQHouse.setShareParams('测试分享标题', '测试分享内容', 'http://img2.imgtn.bdimg.com/it/u=1620444099,390142007&fm=21&gp=0.jpg', 'http://house.qq.com/mobile/index.htm')" ]
            },
            {   n: 'setNavigationBackType',
                iiv:  '2.3',
                idv:  '999',
                aiv:  '2.3',
                adv:  '999',
                doc:  ['{int} 0/1, 0表示首先返回上一个h5页面，如没有则返回上一个native页面, 1表示直接返回上一个native页面'],
                test: ["window.QQHouse.setNavigationBackType(0)", "window.QQHouse.setNavigationBackType(1)"]
            },
            { n:    'getLocationServiceStatus',
              iiv:  '2.5',
              idv:  '999',
              aiv:  '2.5',
              adv:  '999',
              doc:  ['{function} callback 回调函数参数：status授权状态: 0为不允许, 1为允许'],
              test: ["window.QQHouse.getLocationServiceStatus(function(status) { alert('location service status: '+status); })"]
            },
            { n:    'getGPSLocation',
              iiv:  '2.5',
              idv:  '999',
              aiv:  '2.5',
              adv:  '999',
              doc:  ['{function} callback 回调函数参数：code: 0为正常返回结果,其他为获取gps数据异常; lat: 纬度; lng: 经度'],
              test: ["window.QQHouse.getGPSLocation(function(code, lat, lng) { if (code == 0) alert('get location(lat: '+lat+', lng: '+lng+')'); else alert('failed with code: '+code) })"]
            },
    ];

    var house_apis = {};
    if (typeof w[x] !== 'object') {
        config.forEach(function(item) {
            var n = item['n'];
            var iv = item[client_info.platform + 'iv'];
            var dv = item[client_info.platform + 'dv'];
            var doc = item['doc'];
            var test = item['test'];
            if (n && iv && versionGreaterThanEqual(client_info.version, iv)
                    && dv && versionLessThan(client_info.version, dv)) {
                var api = function() {
                    var params = [ client_info.platform, n ].concat(slice
                            .apply(arguments));
                    return callNative.apply(house_apis, params);
                };
                api.iv = iv;
                api.dv = dv;
                api.doc = doc;
                api.test = test;
                api.isApi = true;
                house_apis[n] = api;
            }
        });
    }
    house_apis.javascriptBridgeCallBack = function() {
        var params = slice.apply(arguments);
        var index = Number(params.shift());
        callbackQueue[index].apply(this, params[0]);
    };
    house_apis.democallback = function() {
        if (arguments[0] && typeof arguments[0] === 'object'
                && arguments[0].constructor === Object) {
            var str = '{';
            for (name in arguments[0]) {
                if (typeof arguments[0][name] !== 'function'
                        && arguments[0].hasOwnProperty(name)) {
                    str += name + " : " + arguments[0][name] + "\n";
                }
            }
            alert('callback alert: ' + str + '}');
        } else {
            alert('callback alert:' + slice.apply(arguments));
        }
    };
    house_apis.client_info = client_info;
    w[x] = house_apis;
})(this, 'QQHouse');
